@@ -90,6 +90,9 @@ DataManagerClassifier <- R6::R6Class(
       # Checking Prerequisites---------------------------------------------------
       check_all_args(get_called_args(n = 1L))
 
+      # Set cache dir
+      private$cache_dir <- file.path(create_and_get_tmp_dir(), paste0("dm_", generate_id(5L)))
+
       # Create Dataset-------------------------------------------------------
       private$prepare_datasets(
         data_embeddings = data_embeddings,
@@ -504,6 +507,11 @@ DataManagerClassifier <- R6::R6Class(
     #-----------------------------------------------------------------------------
   ),
   private = list(
+    cache_dir = NULL,
+    finalize = function() {
+      unlink(x = private$cache_dir, recursive = TRUE)
+      gc()
+    },
     prepare_datasets = function(data_embeddings, data_targets, trace) {
       if (inherits(data_embeddings, "EmbeddedText")) {
         data_set_embeddings <- data_embeddings$convert_to_LargeDataSetForTextEmbeddings()
@@ -597,7 +605,7 @@ DataManagerClassifier <- R6::R6Class(
           ),
           load_from_cache_file = FALSE,
           keep_in_memory = FALSE,
-          cache_file_name = file.path(create_and_get_tmp_dir(), generate_id(15L))
+          cache_file_name = file.path(private$cache_dir, generate_id(15L))
         )
         return(dataset)
       } else {
@@ -611,7 +619,7 @@ DataManagerClassifier <- R6::R6Class(
           fn_kwargs = reticulate::dict(list(num_classes = as.integer(self$config$n_classes))),
           load_from_cache_file = FALSE,
           keep_in_memory = FALSE,
-          cache_file_name = file.path(create_and_get_tmp_dir(), generate_id(15L))
+          cache_file_name = file.path(private$cache_dir, generate_id(15L))
         )
         return(dataset)
       } else {
