@@ -33,6 +33,7 @@ TEClassifierParallel <- R6::R6Class(
     #' @param feature_extractor `r get_param_doc_desc("feature_extractor")`
     #' @param target_levels `r get_param_doc_desc("target_levels")`
     #' @param shared_feat_layer `r get_param_doc_desc("shared_feat_layer")`
+    #' @param cls_head_type `r get_param_doc_desc("cls_head_type")`
     #' @param feat_act_fct `r get_param_doc_desc("feat_act_fct")`
     #' @param feat_size `r get_param_doc_desc("feat_size")`
     #' @param feat_bias `r get_param_doc_desc("feat_bias")`
@@ -75,6 +76,7 @@ TEClassifierParallel <- R6::R6Class(
     #' @param tf_bias `r get_param_doc_desc("tf_bias")`
     #' @param tf_parametrizations `r get_param_doc_desc("tf_parametrizations")`
     #' @param tf_normalization_type `r get_param_doc_desc("tf_normalization_type")`
+    #' @param tf_normalization_position `r get_param_doc_desc("tf_normalization_position")`
     #' @param tf_residual_type `r get_param_doc_desc("tf_residual_type")`
     #' @param merge_attention_type `r get_param_doc_desc("merge_attention_type")`
     #' @param merge_num_heads `r get_param_doc_desc("merge_num_heads")`
@@ -88,6 +90,7 @@ TEClassifierParallel <- R6::R6Class(
                          feature_extractor = NULL,
                          target_levels = NULL,
                          shared_feat_layer = TRUE,
+                         cls_head_type = "Regular",
                          feat_act_fct = "ELU",
                          feat_size = 50L,
                          feat_bias = TRUE,
@@ -130,6 +133,7 @@ TEClassifierParallel <- R6::R6Class(
                          tf_bias = FALSE,
                          tf_parametrizations = "None",
                          tf_normalization_type = "LayerNorm",
+                         tf_normalization_position = "Pre",
                          tf_residual_type = "ResidualGate",
                          merge_attention_type = "multi_head",
                          merge_num_heads = 1L,
@@ -153,6 +157,7 @@ TEClassifierParallel <- R6::R6Class(
         n_target_levels = as.integer(length(private$model_config$target_levels)),
         pad_value = as.integer(private$text_embedding_model$pad_value),
         shared_feat_layer = private$model_config$shared_feat_layer,
+        cls_type = private$model_config$cls_head_type,
         feat_act_fct = private$model_config$feat_act_fct,
         feat_size = as.integer(private$model_config$feat_size),
         feat_bias = private$model_config$feat_bias,
@@ -195,6 +200,7 @@ TEClassifierParallel <- R6::R6Class(
         tf_bias = private$model_config$tf_bias,
         tf_parametrizations = private$model_config$tf_parametrizations,
         tf_normalization_type = private$model_config$tf_normalization_type,
+        tf_normalization_position = private$model_config$tf_normalization_position,
         tf_residual_type = private$model_config$tf_residual_type,
         merge_attention_type = private$model_config$merge_attention_type,
         merge_normalization_type = private$model_config$merge_normalization_type,
@@ -205,18 +211,21 @@ TEClassifierParallel <- R6::R6Class(
     },
     #--------------------------------------------------------------------------
     check_param_combinations_configuration = function() {
+      if (private$model_config$feat_size < private$model_config$merge_pooling_features) {
+        warning("merge_pooling_features must be equal or lower as feat_size. Set merge_pooling_features=feat_size.")
+        private$model_config$merge_pooling_features <- private$model_config$feat_size
+      }
+
       if (private$model_config$rec_n_layers == 1L && private$model_config$rec_dropout > 0.0) {
         print_message(
-          msg = "Dropout for recurrent requires at least two layers. Setting rec_dropout to 0.0.",
+          msg = "Dropout for recurrent layers requires at least two layers. Setting rec_dropout to 0.0.",
           trace = TRUE
         )
         private$model_config$rec_dropout <- 0.0
       }
     },
     #--------------------------------------------------------------------------
-    adjust_configuration = function() {
-
-    }
+    adjust_configuration = function() {}
   )
 )
 
