@@ -34,6 +34,7 @@ TEClassifierSequential <- R6::R6Class(
     #' @param skip_connection_type `r get_param_doc_desc("skip_connection_type")`
     #' @param cls_pooling_features `r get_param_doc_desc("cls_pooling_features")`
     #' @param cls_pooling_type `r get_param_doc_desc("cls_pooling_type")`
+    #' @param cls_head_type `r get_param_doc_desc("cls_head_type")`
     #' @param feat_act_fct `r get_param_doc_desc("feat_act_fct")`
     #' @param feat_size `r get_param_doc_desc("feat_size")`
     #' @param feat_bias `r get_param_doc_desc("feat_bias")`
@@ -76,6 +77,7 @@ TEClassifierSequential <- R6::R6Class(
     #' @param tf_bias `r get_param_doc_desc("tf_bias")`
     #' @param tf_parametrizations `r get_param_doc_desc("tf_parametrizations")`
     #' @param tf_normalization_type `r get_param_doc_desc("tf_normalization_type")`
+    #' @param tf_normalization_position `r get_param_doc_desc("tf_normalization_position")`
     #' @param tf_residual_type `r get_param_doc_desc("tf_residual_type")`
     #' @return Function does nothing return. It modifies the current object.
     configure = function(name = NULL,
@@ -86,6 +88,7 @@ TEClassifierSequential <- R6::R6Class(
                          skip_connection_type = "ResidualGate",
                          cls_pooling_features = NULL,
                          cls_pooling_type = "MinMax",
+                         cls_head_type = "Regular",
                          feat_act_fct = "ELU",
                          feat_size = 50L,
                          feat_bias = TRUE,
@@ -128,6 +131,7 @@ TEClassifierSequential <- R6::R6Class(
                          tf_bias = FALSE,
                          tf_parametrizations = "None",
                          tf_normalization_type = "LayerNorm",
+                         tf_normalization_position = "Pre",
                          tf_residual_type = "ResidualGate") {
       private$do_configuration(args = get_called_args(n = 1L))
     }
@@ -148,6 +152,7 @@ TEClassifierSequential <- R6::R6Class(
         skip_connection_type = private$model_config$skip_connection_type,
         cls_pooling_features = as.integer(private$model_config$cls_pooling_features),
         cls_pooling_type = private$model_config$cls_pooling_type,
+        cls_type = private$model_config$cls_head_type,
         feat_act_fct = private$model_config$feat_act_fct,
         feat_size = as.integer(private$model_config$feat_size),
         feat_bias = private$model_config$feat_bias,
@@ -190,23 +195,27 @@ TEClassifierSequential <- R6::R6Class(
         tf_bias = private$model_config$tf_bias,
         tf_parametrizations = private$model_config$tf_parametrizations,
         tf_normalization_type = private$model_config$tf_normalization_type,
+        tf_normalization_position = private$model_config$tf_normalization_position,
         tf_residual_type = private$model_config$tf_residual_type
       )
     },
     #--------------------------------------------------------------------------
     check_param_combinations_configuration = function() {
+      if (private$model_config$feat_size < private$model_config$cls_pooling_features) {
+        warning("cls_pooling_features must be equal or lower as feat_size. Set cls_pooling_features=feat_size.")
+        private$model_config$cls_pooling_features <- private$model_config$feat_size
+      }
+
       if (private$model_config$rec_n_layers == 1L && private$model_config$rec_dropout > 0.0) {
         print_message(
-          msg = "Dropout for recurrent requires at least two layers. Setting rec_dropout to 0.0.",
+          msg = "Dropout for recurrent layers requires at least two layers. Setting rec_dropout to 0.0.",
           trace = TRUE
         )
         private$model_config$rec_dropout <- 0.0
       }
     },
     #--------------------------------------------------------------------------
-    adjust_configuration = function() {
-
-    }
+    adjust_configuration = function() {}
   )
 )
 
