@@ -85,9 +85,11 @@ TEClassifierSequentialPrototype <- R6::R6Class(
     #' @param tf_bias `r get_param_doc_desc("tf_bias")`
     #' @param tf_parametrizations `r get_param_doc_desc("tf_parametrizations")`
     #' @param tf_normalization_type `r get_param_doc_desc("tf_normalization_type")`
+    #' @param tf_normalization_position `r get_param_doc_desc("tf_normalization_position")`
     #' @param tf_residual_type `r get_param_doc_desc("tf_residual_type")`
     #' @param metric_type `r get_param_doc_desc("metric_type")`
     #' @param embedding_dim `r get_param_doc_desc("embedding_dim")`
+    #' @param projection_type `r get_param_doc_desc("projection_type")`
     #' @return Function does nothing return. It modifies the current object.
     configure = function(name = NULL,
                          label = NULL,
@@ -97,6 +99,7 @@ TEClassifierSequentialPrototype <- R6::R6Class(
                          skip_connection_type = "ResidualGate",
                          cls_pooling_features = 50L,
                          cls_pooling_type = "MinMax",
+                         projection_type = "Regular",
                          metric_type = "Euclidean",
                          feat_act_fct = "ELU",
                          feat_size = 50L,
@@ -140,6 +143,7 @@ TEClassifierSequentialPrototype <- R6::R6Class(
                          tf_bias = FALSE,
                          tf_parametrizations = "None",
                          tf_normalization_type = "LayerNorm",
+                         tf_normalization_position = "Pre",
                          tf_residual_type = "ResidualGate",
                          embedding_dim = 2L) {
       arguments <- get_called_args(n = 1L)
@@ -162,6 +166,7 @@ TEClassifierSequentialPrototype <- R6::R6Class(
         skip_connection_type = private$model_config$skip_connection_type,
         cls_pooling_features = as.integer(private$model_config$cls_pooling_features),
         cls_pooling_type = private$model_config$cls_pooling_type,
+        projection_type = private$model_config$projection_type,
         metric_type = private$model_config$metric_type,
         feat_act_fct = private$model_config$feat_act_fct,
         feat_size = as.integer(private$model_config$feat_size),
@@ -205,6 +210,7 @@ TEClassifierSequentialPrototype <- R6::R6Class(
         tf_bias = private$model_config$tf_bias,
         tf_parametrizations = private$model_config$tf_parametrizations,
         tf_normalization_type = private$model_config$tf_normalization_type,
+        tf_normalization_position = private$model_config$tf_normalization_position,
         tf_residual_type = private$model_config$tf_residual_type,
         embedding_dim = as.integer(private$model_config$embedding_dim),
         core_net_type = private$model_config$core_net_type
@@ -214,17 +220,21 @@ TEClassifierSequentialPrototype <- R6::R6Class(
     },
     #--------------------------------------------------------------------------
     check_param_combinations_configuration = function() {
+      if (private$model_config$feat_size < private$model_config$cls_pooling_features) {
+        warning("cls_pooling_features must be equal or lower as feat_size. Set cls_pooling_features=feat_size.")
+        private$model_config$cls_pooling_features <- private$model_config$feat_size
+      }
+
       if (private$model_config$rec_n_layers == 1L && private$model_config$rec_dropout > 0.0) {
         print_message(
-          msg = "Dropout for recurrent requires at least two layers. Setting rec_dropout to 0.0.",
+          msg = "Dropout for recurrent layers requires at least two layers. Setting rec_dropout to 0.0.",
           trace = TRUE
         )
         private$model_config$rec_dropout <- 0.0
       }
     },
     #--------------------------------------------------------------------------
-    adjust_configuration = function() {
-    }
+    adjust_configuration = function() {}
   )
 )
 
