@@ -18,12 +18,12 @@ import math
 import safetensors
 
 class TEClassifierSequential(torch.nn.Module):
-  def __init__(self,times, features, cls_pooling_features, pad_value,n_target_levels,inc_cls_head=True,skip_connection_type="ResidualGate",cls_type="regular",cls_pooling_type="MinMax", 
+  def __init__(self,times, features, cls_pooling_features, pad_value,n_target_levels,inc_cls_head=True,skip_connection_type="ResidualGate",cls_type="Regular",cls_pooling_type="MinMax", 
               feat_act_fct="ELU",feat_size=50,feat_bias=True,feat_dropout=0.0,feat_parametrizations="None",feat_normalization_type="LayerNorm",
               ng_conv_act_fct="ELU",ng_conv_n_layers=0,ng_conv_ks_min=2, ng_conv_ks_max=4,ng_conv_dropout=0.1, ng_conv_bias=False, ng_conv_parametrizations="None", ng_conv_residual_type="ResidualGate",ng_conv_normalization_type="LayerNorm",
               dense_act_fct="ELU",dense_n_layers=0,dense_dropout=0.0,dense_bias=False,dense_parametrizations="None", dense_residual_type="ResidualGate",dense_normalization_type="LayerNorm",
               rec_act_fct="Tanh",rec_n_layers=0,rec_type="GRU",rec_bidirectional=False,rec_dropout=0.0,rec_bias=False,rec_parametrizations="None",rec_residual_type="ResidualGate",rec_normalization_type="LayerNorm", 
-              tf_act_fct="ELU",tf_dense_dim=50,tf_n_layers=0,tf_dropout_rate_1=0.0,tf_dropout_rate_2=0.0,tf_attention_type="MultiHead",tf_positional_type ="absolute",tf_num_heads=1,tf_bias=False,tf_parametrizations="None",tf_residual_type="ResidualGate",tf_normalization_type="LayerNorm",
+              tf_act_fct="ELU",tf_dense_dim=50,tf_n_layers=0,tf_dropout_rate_1=0.0,tf_dropout_rate_2=0.0,tf_attention_type="MultiHead",tf_positional_type ="absolute",tf_num_heads=1,tf_bias=False,tf_parametrizations="None",tf_residual_type="ResidualGate",tf_normalization_type="LayerNorm", tf_normalization_position="pre",
               device=None, dtype=None):
       super().__init__()
       self.inc_cls_head=inc_cls_head
@@ -67,6 +67,7 @@ class TEClassifierSequential(torch.nn.Module):
           bias=tf_bias,
           parametrizations=tf_parametrizations,
           normalization_type=tf_normalization_type,
+          normalization_position=tf_normalization_position,
           device=device, 
           dtype=dtype,
           residual_type=tf_residual_type
@@ -145,10 +146,26 @@ class TEClassifierSequential(torch.nn.Module):
       self.residual_connection=layer_residual_connection(skip_connection_type,pad_value)  
       
       if inc_cls_head==True:
-        if cls_type=="regular":
+        if cls_type=="Regular":
           self.classification_head=torch.nn.Linear(
                 in_features=cls_pooling_features,
                 out_features=n_target_levels)
+        elif cls_type=="PairwiseOrthogonal":
+          self.classification_head=pairwise_orthogonal_dense(
+                input_size=cls_pooling_features,
+                output_size=n_target_levels,
+                bias=False,
+                pre_dense=False,
+                device=device, 
+                dtype=dtype)
+        elif cls_type=="PairwiseOrthogonalDense":
+          self.classification_head=pairwise_orthogonal_dense(
+                input_size=cls_pooling_features,
+                output_size=n_target_levels,
+                bias=False,
+                pre_dense=True,
+                device=device, 
+                dtype=dtype)
 
   def forward(self,x,prediction_mode=True):
     y_original=self.masking_layer(x)
@@ -174,7 +191,7 @@ class TEClassifierParallel(torch.nn.Module):
               ng_conv_act_fct="ELU",ng_conv_n_layers=0,ng_conv_ks_min=2, ng_conv_ks_max=4,ng_conv_dropout=0.1, ng_conv_bias=False, ng_conv_parametrizations="None", ng_conv_residual_type="ResidualGate",ng_conv_normalization_type="LayerNorm",
               dense_act_fct="ELU",dense_n_layers=0,dense_dropout=0.0,dense_bias=False,dense_parametrizations="None", dense_residual_type="ResidualGate",dense_normalization_type="LayerNorm",
               rec_act_fct="Tanh",rec_n_layers=0,rec_type="GRU",rec_bidirectional=False,rec_dropout=0.0,rec_bias=False,rec_parametrizations="None", rec_residual_type="ResidualGate",rec_normalization_type="LayerNorm",
-              tf_act_fct="ELU",tf_dense_dim=50,tf_n_layers=0,tf_dropout_rate_1=0.0,tf_dropout_rate_2=0.0,tf_attention_type="MultiHead",tf_positional_type ="absolute",tf_num_heads=1,tf_bias=False,tf_parametrizations="None",tf_residual_type="ResidualGate",tf_normalization_type="LayerNorm",
+              tf_act_fct="ELU",tf_dense_dim=50,tf_n_layers=0,tf_dropout_rate_1=0.0,tf_dropout_rate_2=0.0,tf_attention_type="MultiHead",tf_positional_type ="absolute",tf_num_heads=1,tf_bias=False,tf_parametrizations="None",tf_residual_type="ResidualGate",tf_normalization_type="LayerNorm", tf_normalization_position="pre",
               merge_attention_type="MultiHead",merge_num_heads=1,merge_normalization_type="LayerNorm",merge_pooling_type="MinMax",merge_pooling_features=2,
               device=None, dtype=None):
       super().__init__()
@@ -220,6 +237,7 @@ class TEClassifierParallel(torch.nn.Module):
           bias=tf_bias,
           parametrizations=tf_parametrizations,
           normalization_type=tf_normalization_type,
+          normalization_position=tf_normalization_position,
           pad_value=pad_value,
           device=device, 
           dtype=dtype,
@@ -391,10 +409,26 @@ class TEClassifierParallel(torch.nn.Module):
       )
       
       if inc_cls_head==True:
-        if cls_type=="regular":
+        if cls_type=="Regular":
           self.classification_head=torch.nn.Linear(
                 in_features=merge_pooling_features,
                 out_features=n_target_levels)
+        elif cls_type=="PairwiseOrthogonal":
+          self.classification_head=pairwise_orthogonal_dense(
+                input_size=merge_pooling_features,
+                output_size=n_target_levels,
+                bias=False,
+                pre_dense=False,
+                device=device, 
+                dtype=dtype)
+        elif cls_type=="PairwiseOrthogonalDense":
+          self.classification_head=pairwise_orthogonal_dense(
+                input_size=merge_pooling_features,
+                output_size=n_target_levels,
+                bias=False,
+                pre_dense=True,
+                device=device, 
+                dtype=dtype)
 
   def forward(self,x,prediction_mode=True):
     y=self.masking_layer(x)
@@ -445,12 +479,12 @@ class TEClassifierParallel(torch.nn.Module):
 
 
 class TEClassifierPrototype(torch.nn.Module):
-  def __init__(self,times, features, pad_value,target_levels,core_net_type,embedding_dim=2,skip_connection_type="ResidualGate",inc_cls_head=True,cls_type="regular", 
+  def __init__(self,times, features, pad_value,target_levels,core_net_type,projection_type="Regular",embedding_dim=2,skip_connection_type="ResidualGate",inc_cls_head=True,cls_type="regular", 
               shared_feat_layer=True, feat_act_fct="ELU",feat_size=50,feat_bias=True,feat_dropout=0.0,feat_parametrizations="None",feat_normalization_type="LayerNorm",
               ng_conv_act_fct="ELU",ng_conv_n_layers=0,ng_conv_ks_min=2, ng_conv_ks_max=4,ng_conv_dropout=0.1, ng_conv_bias=False, ng_conv_parametrizations="None", ng_conv_residual_type="ResidualGate",ng_conv_normalization_type="LayerNorm",
               dense_act_fct="ELU",dense_n_layers=0,dense_dropout=0.0,dense_bias=False,dense_parametrizations="None", dense_residual_type="ResidualGate",dense_normalization_type="LayerNorm",
               rec_act_fct="Tanh",rec_n_layers=0,rec_type="GRU",rec_bidirectional=False,rec_dropout=0.0,rec_bias=False,rec_parametrizations="None", rec_residual_type="ResidualGate",rec_normalization_type="LayerNorm",
-              tf_act_fct="ELU",tf_dense_dim=50,tf_n_layers=0,tf_dropout_rate_1=0.0,tf_dropout_rate_2=0.0,tf_attention_type="MultiHead",tf_positional_type ="absolute",tf_num_heads=1,tf_bias=False,tf_parametrizations="None",tf_residual_type="ResidualGate",tf_normalization_type="LayerNorm",
+              tf_act_fct="ELU",tf_dense_dim=50,tf_n_layers=0,tf_dropout_rate_1=0.0,tf_dropout_rate_2=0.0,tf_attention_type="MultiHead",tf_positional_type ="absolute",tf_num_heads=1,tf_bias=False,tf_parametrizations="None",tf_residual_type="ResidualGate",tf_normalization_type="LayerNorm", tf_normalization_position="pre",
               merge_attention_type="MultiHead",merge_num_heads=1,merge_normalization_type="LayerNorm",merge_pooling_features=2,merge_pooling_type="MinMax",
               cls_pooling_features=2,cls_pooling_type="MinMax",
               metric_type="Euclidean",
@@ -513,6 +547,7 @@ class TEClassifierPrototype(torch.nn.Module):
         tf_parametrizations=tf_parametrizations,
         tf_residual_type=tf_residual_type,
         tf_normalization_type=tf_normalization_type,
+        tf_normalization_position=tf_normalization_position,
         device=device, 
         dtype=dtype
       )
@@ -569,6 +604,7 @@ class TEClassifierPrototype(torch.nn.Module):
         tf_parametrizations=tf_parametrizations,
         tf_residual_type=tf_residual_type,
         tf_normalization_type=tf_normalization_type,
+        tf_normalization_position=tf_normalization_position,
         device=device, 
         dtype=dtype,
         merge_attention_type=merge_attention_type,
@@ -582,17 +618,45 @@ class TEClassifierPrototype(torch.nn.Module):
     
     self.trained_prototypes=torch.ones(1)
     self.class_labels=torch.ones(1)
+    self.projection_type=projection_type
     
-    if core_net_type=="sequential":
-      self.embedding_head=torch.nn.Linear(
-        in_features=cls_pooling_features,
-        out_features=self.embedding_dim,
-        bias=True)
-    elif core_net_type=="parallel":
-      self.embedding_head=torch.nn.Linear(
-        in_features=merge_pooling_features,
-        out_features=self.embedding_dim,
-        bias=True)
+    if self.projection_type=="Regular":
+      if core_net_type=="sequential":
+        self.embedding_head=torch.nn.Linear(
+          in_features=cls_pooling_features,
+          out_features=self.embedding_dim,
+          bias=True)
+      elif core_net_type=="parallel":
+        self.embedding_head=torch.nn.Linear(
+          in_features=merge_pooling_features,
+          out_features=self.embedding_dim,
+          bias=True)
+    elif self.projection_type=="PairwiseOrthogonal":
+      if core_net_type=="sequential":
+        self.embedding_head=pairwise_orthogonal_dense(
+          input_size=cls_pooling_features,
+          output_size=self.embedding_dim,
+          pre_dense=False,
+          bias=True)
+      elif core_net_type=="parallel":
+        self.embedding_head=pairwise_orthogonal_dense(
+          input_size=merge_pooling_features,
+          output_size=self.embedding_dim,
+          pre_dense=False,
+          bias=True)
+    elif self.projection_type=="PairwiseOrthogonalDense":
+      if core_net_type=="sequential":
+        self.embedding_head=pairwise_orthogonal_dense(
+          input_size=cls_pooling_features,
+          output_size=self.embedding_dim,
+          pre_dense=True,
+          bias=True)
+      elif core_net_type=="parallel":
+        self.embedding_head=pairwise_orthogonal_dense(
+          input_size=merge_pooling_features,
+          output_size=self.embedding_dim,
+          pre_dense=True,
+          bias=True)          
     
     self.class_mean=layer_class_mean()
     self.metric=layer_protonet_metric(metric_type=metric_type)
