@@ -5,6 +5,9 @@ testthat::skip_if_not(
   message = "Necessary python modules not available"
 )
 
+# Start time
+test_time_start <- Sys.time()
+
 # SetUp-------------------------------------------------------------------------
 root_path_general_data <- testthat::test_path("test_data/Embeddings")
 
@@ -56,3 +59,9 @@ for (aug_num in aug_nums) {
     }
   })
 }
+
+# Monitor test time
+monitor_test_time_on_CI(
+  start_time = test_time_start,
+  test_name = "01_05_KNNOR"
+)
