@@ -1,3 +1,6 @@
+# Start time
+test_time_start <- Sys.time()
+
 test_that("logging in R - state log", {
   dir <- testthat::test_path("test_artefacts")
   create_dir(dir, FALSE)
@@ -43,3 +46,9 @@ test_that("logging in R - loss log", {
   expect_equal(colnames(res), c("train", "validation", "test", "epoch"))
   expect_equal(unname(as.matrix(res)[1, 1:3]), c(-100, -100, -100))
 })
+
+# Monitor test time
+monitor_test_time_on_CI(
+  start_time = test_time_start,
+  test_name = "01_06_logging"
+)
