@@ -4,6 +4,9 @@ testthat::skip_if_not(
   message = "Necessary python modules not available"
 )
 
+# Start time
+test_time_start <- Sys.time()
+
 # Load python scripts
 load_all_py_scripts()
 
@@ -14,6 +17,8 @@ test_that("stack_dense_layer", {
   times <- sample(x = seq(from = 3, to = 10, by = 1), size = 1)
   features <- sample(x = seq(from = 3, to = 1024, by = 1), size = 1)
   sequence_length <- sample(x = seq(from = 1, to = times, by = 1), size = 30, replace = TRUE)
+
+
   example_tensor <- generate_tensors(
     times = times,
     features = features,
@@ -177,6 +182,9 @@ test_that("stack_tf_encoder_layer", {
   masking_layer <- py$masking_layer(pad_value)
   values <- masking_layer(example_tensor)
 
+  normalization_positions <- c("Post", "Pre")
+  residual_types <- c("None", "Addition", "ResidualGate")
+
   # Test for equal, more, and fewer features as input size
   features_output <- c(
     features,
@@ -185,56 +193,61 @@ test_that("stack_tf_encoder_layer", {
   )
 
   for (attention_type in attention_types) {
-    layer <- py$stack_tf_encoder_layer(
-      dense_dim = as.integer(4 * features),
-      attention_type = attention_type,
-      num_heads = 2L,
-      times = as.integer(times),
-      features = as.integer(features),
-      n_layers = 3L,
-      dropout_rate_1 = 0.1,
-      dropout_rate_2 = 0.2,
-      pad_value = as.integer(pad_value),
-      positional_embedding = "absolute",
-      bias = TRUE,
-      parametrizations = "None",
-      dtype = values[[1]]$dtype,
-      device = device,
-      residual_type = "ResidualGate"
-    )$to(device)
-    layer$eval()
+    for (normalization_position in normalization_positions) {
+      for (residual_type in residual_types) {
+        layer <- py$stack_tf_encoder_layer(
+          dense_dim = as.integer(4 * features),
+          attention_type = attention_type,
+          num_heads = 2L,
+          times = as.integer(times),
+          features = as.integer(features),
+          n_layers = 3L,
+          dropout_rate_1 = 0.1,
+          dropout_rate_2 = 0.2,
+          pad_value = as.integer(pad_value),
+          positional_embedding = "absolute",
+          normalization_position = normalization_position,
+          bias = TRUE,
+          parametrizations = "None",
+          dtype = values[[1]]$dtype,
+          device = device,
+          residual_type = residual_type
+        )$to(device)
+        layer$eval()
 
-    y <- layer(
-      x = values[[1]],
-      seq_len = values[[2]],
-      mask_times = values[[3]],
-      mask_features = values[[4]]
-    )
+        y <- layer(
+          x = values[[1]],
+          seq_len = values[[2]],
+          mask_times = values[[3]],
+          mask_features = values[[4]]
+        )
 
-    # Test that masking values are the same
-    expect_equal(tensor_to_numpy(y[[2]]), tensor_to_numpy(values[[2]]))
-    expect_equal(tensor_to_numpy(y[[3]]), tensor_to_numpy(values[[3]]))
+        # Test that masking values are the same
+        expect_equal(tensor_to_numpy(y[[2]]), tensor_to_numpy(values[[2]]))
+        expect_equal(tensor_to_numpy(y[[3]]), tensor_to_numpy(values[[3]]))
 
-    # Test that padding is not destroyed
-    y_2 <- masking_layer(y[[1]])
-    expect_equal(tensor_to_numpy(y[[2]]), tensor_to_numpy(y_2[[2]]))
-    expect_equal(tensor_to_numpy(y[[3]]), tensor_to_numpy(y_2[[3]]))
-    expect_equal(tensor_to_numpy(y[[4]]), tensor_to_numpy(y_2[[4]]))
+        # Test that padding is not destroyed
+        y_2 <- masking_layer(y[[1]])
+        expect_equal(tensor_to_numpy(y[[2]]), tensor_to_numpy(y_2[[2]]))
+        expect_equal(tensor_to_numpy(y[[3]]), tensor_to_numpy(y_2[[3]]))
+        expect_equal(tensor_to_numpy(y[[4]]), tensor_to_numpy(y_2[[4]]))
 
-    # Test that values do not change at random for same input
-    y_1 <- layer(
-      x = values[[1]],
-      seq_len = values[[2]],
-      mask_times = values[[3]],
-      mask_features = values[[4]]
-    )
-    y_2 <- layer(
-      x = values[[1]],
-      seq_len = values[[2]],
-      mask_times = values[[3]],
-      mask_features = values[[4]]
-    )
-    expect_equal(tensor_to_numpy(y_1[[1]]), tensor_to_numpy(y_2[[1]]))
+        # Test that values do not change at random for same input
+        y_1 <- layer(
+          x = values[[1]],
+          seq_len = values[[2]],
+          mask_times = values[[3]],
+          mask_features = values[[4]]
+        )
+        y_2 <- layer(
+          x = values[[1]],
+          seq_len = values[[2]],
+          mask_times = values[[3]],
+          mask_features = values[[4]]
+        )
+        expect_equal(tensor_to_numpy(y_1[[1]]), tensor_to_numpy(y_2[[1]]))
+      }
+    }
   }
 })
 
@@ -317,3 +330,9 @@ test_that("stack_n_gram_convolution", {
     expect_equal(dim(tensor_to_numpy(y_1[[1]])), c(length(sequence_length), times, features))
   }
 })
+
+# Monitor test time
+monitor_test_time_on_CI(
+  start_time = test_time_start,
+  test_name = "02_03_pytorch_stack_layers"
+)
