@@ -4,6 +4,9 @@ testthat::skip_if_not(
   message = "Necessary python modules not available"
 )
 
+# Start time
+test_time_start <- Sys.time()
+
 # SetUp Test---------------------------------------------------------------------
 root_path_data <- testthat::test_path("test_data/LargeDataSetForTexts")
 create_dir(testthat::test_path("test_artefacts"), FALSE)
@@ -807,3 +810,9 @@ if (dir.exists(root_path_results)) {
     recursive = TRUE
   )
 }
+
+# Monitor test time
+monitor_test_time_on_CI(
+  start_time = test_time_start,
+  test_name = "03_01_LargeDataSetForTexts"
+)
