@@ -4,6 +4,9 @@ testthat::skip_if_not(
   message = "Necessary python modules not available"
 )
 
+# Start time
+test_time_start <- Sys.time()
+
 load_all_py_scripts()
 
 # SetUp Test---------------------------------------------------------------------
@@ -218,3 +221,9 @@ if (dir.exists(root_path_results)) {
     recursive = TRUE
   )
 }
+
+# Monitor test time
+monitor_test_time_on_CI(
+  start_time = test_time_start,
+  test_name = "03_02_LargeDataSetForTextEmbeddings"
+)
