@@ -4,6 +4,9 @@ testthat::skip_if_not(
   message = "Necessary python modules not available"
 )
 
+# Start time
+test_time_start <- Sys.time()
+
 # SetUp Test---------------------------------------------------------------------
 root_path_general_data <- testthat::test_path("test_data/Embeddings")
 # root_path_data <- testthat::test_path("test_data/EmbeddedText")
@@ -106,3 +109,9 @@ test_that("EmbeddedText - No FeatureExtractor", {
   # Correct padding value
   expect_equal(new_data_set_converted$get_pad_value(), -100)
 })
+
+# Monitor test time
+monitor_test_time_on_CI(
+  start_time = test_time_start,
+  test_name = "03_03_EmbeddedText"
+)
