@@ -4,6 +4,9 @@ testthat::skip_if_not(
   message = "Necessary python modules not available"
 )
 
+# Start time
+test_time_start <- Sys.time()
+
 # SetUp-------------------------------------------------------------------------
 root_path_general_data <- testthat::test_path("test_data/Embeddings")
 root_path_data <- testthat::test_path("test_data/classifier")
@@ -237,3 +240,9 @@ for (method in methods) {
     }
   }
 }
+
+# Monitor test time
+monitor_test_time_on_CI(
+  start_time = test_time_start,
+  test_name = "03_04_DataManager"
+)
